@@ -292,7 +292,7 @@ compute.ini <- function(r, parameter) {
     ka_ini <- lm(log(y) ~  time, data=subset(abs, y>0))$coefficients[[2]]
     if (ka_ini < 0)
       ka_ini <- 1
-#    ka_ini <- lm(y ~ -1 + time, data=subset(abs, y>0))$coefficients[[1]]
+    #    ka_ini <- lm(y ~ -1 + time, data=subset(abs, y>0))$coefficients[[1]]
     Tk0_ini <- mean(tmax)
     if (ka_ini>0)
       V_ini <- 1/mean(ymax)*ka_ini/abs(ka_ini-k_ini)
@@ -317,8 +317,10 @@ compute.ini <- function(r, parameter) {
   Vm_ini <- Cl_ini*(2*Km_ini)
   k12_ini <- k_ini/2
   k21_ini <- k_ini/2
-  k13_ini <- k_ini/2
-  k31_ini <- k_ini/2
+  k13_ini <- k_ini/4
+  k31_ini <- k_ini/4
+  # k13_ini <- k_ini/2
+  # k31_ini <- k_ini/2
   
   list.ini <- c(list.ini, Cl=Cl_ini, Km=Km_ini, Vm=Vm_ini)
   list.ini <- c(list.ini, k12=k12_ini, k21=k21_ini, k13=k13_ini, k31=k31_ini)
@@ -334,8 +336,9 @@ err <-  function(parameter, y, p.ind, N, a) {
   if (any(is.nan(f)) | any(is.infinite(f)))
     e <- Inf
   else
-    e <- mean((log(f+a)-log(y+a))^2)
-  #  return(e)
+  e <- mean((log(f+a)-log(y+a))^2)
+#    e <- mean((f^a-y^a)^2)
+  return(e)
 }
 
 #-------------------------------------------------
@@ -351,15 +354,16 @@ pop.opt <- function(p0) {
   else
     p.ind <- p0
   a <- max(-min(y) + 0.5, 0.5)
+#  if ("k12" %in% names(p0))  browser()
   r <- optim(log(p0), err, y=y, p.ind=p.ind, N=N, a=a)
   return(exp(r$par))
 }
 
 
 #-------------------------------------------------
-compute.bic <- function(parameter, data, new.dir=NULL, level=NULL) {
+compute.bic <- function(parameter, data, new.dir=NULL, level=NULL, par.ini=NULL, linearization=F) {
   cat("\n")
-  r <- pkpopini(parameter=parameter, data=data, new.dir=new.dir) 
+  r <- pkpopini(parameter=parameter, data=data, new.dir=new.dir, par.ini=par.ini) 
   print(r$project)
   mlx.loadProject(projectFile = r$project)
   g=mlx.getObservationInformation()
@@ -385,14 +389,18 @@ compute.bic <- function(parameter, data, new.dir=NULL, level=NULL) {
   }
   if (!("importanceSampling" %in% launched.tasks[["logLikelihoodEstimation"]])) { 
     cat("Estimation of the log-likelihood... \n")
-    mlx.runLogLikelihoodEstimation()
+    mlx.runLogLikelihoodEstimation(linearization=linearization)
   }
   setwd(w.dir)
-  ofv <- mlx.getEstimatedLogLikelihood()[[1]][['OFV']]
+  g <- mlx.getEstimatedLogLikelihood()[[1]]
+  r$ofv <- g[['OFV']]
   # if (is.null(ofv))
   #   ofv <- mlx.getEstimatedLogLikelihood()[[1]][['OFV']]
-  r$bic <- ofv + (2*length(parameter)+2)*log(n)
+  r$bicc <- g[['BICc']]
+  r$bic <- g[['BIC']]
+  r$aic <- g[['AIC']]
   r$pop.est <- mlx.getEstimatedPopulationParameters()
+  print(g)
   return(r)
 }
 
@@ -434,6 +442,7 @@ read.res <- function(file) {
 # Get the separator of a file --------------------------------------------------
 .getDelimiter <- function(fileName, sep = NULL){
   L <- suppressMessages(suppressWarnings(readLines(fileName, n = 1)))
+  L <- gsub("#","", L)
   sepToCheck = c(' ', '\t', ',', ';')
   if (!is.null(sep)) sepToCheck <- unique(c(sep, sepToCheck))
   nSepToCheck <- length(sepToCheck)
@@ -494,3 +503,86 @@ get_lixoft_options <- function() {
   info <- ifelse(op$info == 1, FALSE, TRUE)
   return(list(errors = errors, warnings = warnings, info = info))
 }
+
+############################################################################
+
+def.variable <- function(weight=NULL, prior=NULL, criterion=NULL, fix.param0=NULL, fix.param1=NULL) {
+  go <- mlx.getObservationInformation()
+  y.name <- names(go$mapping)
+  y.type <- go$type[y.name]
+  id <- n <- NULL
+  for (yn in y.name) {
+    id <- unique(c(id, go[[yn]]$id))
+    n <- c(n, nrow(go[[yn]]))
+  }
+  N <- length(id)
+  nc <- n[which(y.type=="continuous")]
+  
+  if (identical(criterion,"AIC")) {
+    pen.coef <- rep(2, length(nc)+2)  
+  } else if (identical(criterion,"BIC")) {
+    pen.coef <- rep(log(N), length(nc)+2)
+  } else if (identical(criterion,"BICc")) {
+    pen.coef <- c(log(N), rep(log(sum(n)), length(nc)+1))
+    #  pen.coef <- c(log(N), log(sum(n)), rep(log(sum(nc)), length(nc)))
+    #    pen.coef <- c(log(N), log(sum(nc)), log(nc))
+  } else {
+    pen.coef <- rep(criterion, length(nc)+2)
+  }
+  
+  if (is.null(weight$is.weight)) { 
+    if (is.null(weight) & is.null(prior))
+      is.weight <- F
+    else
+      is.weight <- T
+  } else {
+    is.weight <- weight$is.weight
+  }
+  
+  w.cov <- weight$covariate
+  if (!is.null(prior$covariate)) 
+    w.cov <- -2*log(prior$covariate/(1-prior$covariate))/pen.coef[1]
+  if (is.null(w.cov))
+    w.cov <- 1
+  if (length(w.cov)==1) {
+    cov.model <- do.call(rbind, mlx.getIndividualParameterModel()$covariateModel)
+    foo <- w.cov
+    w.cov <- cov.model
+    w.cov[] <- foo
+  }
+  
+  w.cor <- weight$correlation
+  if (!is.null(prior$correlation)) 
+    w.cor <- -2*log(prior$correlation/(1-prior$correlation))/pen.coef[1]
+  if (is.null(w.cor))
+    w.cor <- 1
+  if (length(w.cor)==1) {
+    var.model <- mlx.getIndividualParameterModel()$variability$id
+    n.param <- names(which(var.model))
+    d.param <- length(n.param)
+    w.cor <- matrix(w.cor, nrow=d.param, ncol=d.param, dimnames=list(n.param, n.param))
+  }
+  if (!is.null(w.cor))
+    w.cor[fix.param0, ] <- w.cor[, fix.param0] <- 1e10
+  w.cor <- w.cor*lower.tri(w.cor)
+  w.cor[which(is.nan(w.cor))] <- 0
+  
+  if (!is.null(prior$variance)) 
+    w.var <- -2*log(prior$variance/(1-prior$variance))/pen.coef[1]
+  else
+    w.var <- weight$variance 
+  foo <- mlx.getIndividualParameterModel()$variability$id
+  foo[] <- 1
+  if (length(w.var)==1 & is.null(names(w.var)))
+    foo[] <- w.var
+  foo[names(w.var)] <- w.var
+  w.var <- foo
+  # w.var[fix.param0] <- 1e10
+  # w.var[fix.param1] <- 0
+  
+  
+  weight <- list(covariate=w.cov, correlation=w.cor, variance=w.var, is.weight=is.weight)
+  
+  return(list(n=n, N=N, weight=weight, pen.coef=pen.coef))
+}
+
